package org.apache.commons.lang3;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.Serializable;
import java.util.Comparator;
import org.apache.commons.lang3.Validate;

public final class Range<T> implements Serializable {
  private static final long serialVersionUID = 1L;
  
  private final Comparator<T> comparator;
  
  private transient int hashCode;
  
  private final T maximum;
  
  private final T minimum;
  
  private transient String toString;
  
  public static <T extends Comparable<T>> org.apache.commons.lang3.Range<T> between(T fromInclusive, T toInclusive) {
    return between(fromInclusive, toInclusive, null);
  }
  
  public static <T> org.apache.commons.lang3.Range<T> between(T fromInclusive, T toInclusive, Comparator<T> comparator) {
    return new org.apache.commons.lang3.Range<>(fromInclusive, toInclusive, comparator);
  }
  
  public static <T extends Comparable<T>> org.apache.commons.lang3.Range<T> is(T element) {
    return between(element, element, null);
  }
  
  public static <T> org.apache.commons.lang3.Range<T> is(T element, Comparator<T> comparator) {
    return between(element, element, comparator);
  }
  
  private Range(T element1, T element2, Comparator<T> comp) {
    if (element1 == null || element2 == null)
      throw new IllegalArgumentException("Elements in a range must not be null: element1=" + element1 + ", element2=" + element2); 
    if (comp == null) {
      this.comparator = (Comparator<T>)ComparableComparator.INSTANCE;
    } else {
      this.comparator = comp;
    } 
    if (this.comparator.compare(element1, element2) < 1) {
      this.minimum = element1;
      this.maximum = element2;
    } else {
      this.minimum = element2;
      this.maximum = element1;
    } 
  }
  
  public boolean contains(T element) {
    if (element == null)
      return false; 
    return (this.comparator.compare(element, this.minimum) > -1 && this.comparator.compare(element, this.maximum) < 1);
  }
  
  public boolean containsRange(org.apache.commons.lang3.Range<T> otherRange) {
    if (otherRange == null)
      return false; 
    return (contains(otherRange.minimum) && 
      contains(otherRange.maximum));
  }
  
  public int elementCompareTo(T element) {
    Validate.notNull(element, "element", new Object[0]);
    if (isAfter(element))
      return -1; 
    if (isBefore(element))
      return 1; 
    return 0;
  }
  
  public boolean equals(Object obj) {
    if (obj == this)
      return true; 
    if (obj == null || obj.getClass() != getClass())
      return false; 
    org.apache.commons.lang3.Range<T> range = (org.apache.commons.lang3.Range<T>)obj;
    return (this.minimum.equals(range.minimum) && this.maximum
      .equals(range.maximum));
  }
  
  public Comparator<T> getComparator() {
    return this.comparator;
  }
  
  public T getMaximum() {
    return this.maximum;
  }
  
  public T getMinimum() {
    return this.minimum;
  }
  
  public int hashCode() {
    int result = this.hashCode;
    if (this.hashCode == 0) {
      result = 17;
      result = 37 * result + getClass().hashCode();
      result = 37 * result + this.minimum.hashCode();
      result = 37 * result + this.maximum.hashCode();
      this.hashCode = result;
    } 
    return result;
  }
  
  public org.apache.commons.lang3.Range<T> intersectionWith(org.apache.commons.lang3.Range<T> other) {
    if (!isOverlappedBy(other))
      throw new IllegalArgumentException(String.format("Cannot calculate intersection with non-overlapping range %s", new Object[] { other })); 
    if (equals(other))
      return this; 
    T min = (getComparator().compare(this.minimum, other.minimum) < 0) ? other.minimum : this.minimum;
    T max = (getComparator().compare(this.maximum, other.maximum) < 0) ? this.maximum : other.maximum;
    return between(min, max, getComparator());
  }
  
  public boolean isAfter(T element) {
    if (element == null)
      return false; 
    return (this.comparator.compare(element, this.minimum) < 0);
  }
  
  public boolean isAfterRange(org.apache.commons.lang3.Range<T> otherRange) {
    if (otherRange == null)
      return false; 
    return isAfter(otherRange.maximum);
  }
  
  public boolean isBefore(T element) {
    if (element == null)
      return false; 
    return (this.comparator.compare(element, this.maximum) > 0);
  }
  
  public boolean isBeforeRange(org.apache.commons.lang3.Range<T> otherRange) {
    if (otherRange == null)
      return false; 
    return isBefore(otherRange.minimum);
  }
  
  public boolean isEndedBy(T element) {
    if (element == null)
      return false; 
    return (this.comparator.compare(element, this.maximum) == 0);
  }
  
  public boolean isNaturalOrdering() {
    return (this.comparator == ComparableComparator.INSTANCE);
  }
  
  public boolean isOverlappedBy(org.apache.commons.lang3.Range<T> otherRange) {
    if (otherRange == null)
      return false; 
    return (otherRange.contains(this.minimum) || otherRange
      .contains(this.maximum) || 
      contains(otherRange.minimum));
  }
  
  public boolean isStartedBy(T element) {
    if (element == null)
      return false; 
    return (this.comparator.compare(element, this.minimum) == 0);
  }
  
  public T fit(T element) {
    Validate.notNull(element, "element", new Object[0]);
    if (isAfter(element))
      return this.minimum; 
    if (isBefore(element))
      return this.maximum; 
    return element;
  }
  
  public String toString() {
    if (this.toString == null)
      this.toString = "[" + this.minimum + ".." + this.maximum + "]"; 
    return this.toString;
  }
  
  public String toString(String format) {
    return String.format(format, new Object[] { this.minimum, this.maximum, this.comparator });
  }
}
